package com.iqball.app.component

import androidx.compose.foundation.Image
import androidx.compose.foundation.background
import androidx.compose.foundation.layout.Arrangement
import androidx.compose.foundation.layout.Box
import androidx.compose.foundation.layout.Row
import androidx.compose.foundation.layout.fillMaxHeight
import androidx.compose.foundation.layout.fillMaxSize
import androidx.compose.runtime.Composable
import androidx.compose.ui.Alignment
import androidx.compose.ui.Modifier
import androidx.compose.ui.graphics.Color
import androidx.compose.ui.res.painterResource
import com.iqball.app.R
import com.iqball.app.domains.getPlayerInfo
import com.iqball.app.model.tactic.BallComponent
import com.iqball.app.model.tactic.CourtType
import com.iqball.app.model.tactic.PlayerLike
import com.iqball.app.model.tactic.StepContent
import net.engawapg.lib.zoomable.rememberZoomState
import net.engawapg.lib.zoomable.zoomable

@Composable
fun BasketCourt(content: StepContent, type: CourtType) {
    val courtImg = when (type) {
        CourtType.Plain -> R.drawable.plain_court
        CourtType.Half -> R.drawable.half_court
    }

    val zoomState = rememberZoomState()
    val components = content.components

    Row(
        modifier = Modifier
            .background(Color.LightGray)
            .fillMaxSize()
            .zoomable(zoomState),
        horizontalArrangement = Arrangement.Center,
        verticalAlignment = Alignment.CenterVertically
    ) {
        Box {
            Image(
                painter = painterResource(id = courtImg),
                contentDescription = "court",
                modifier = Modifier
                    .background(Color.White)
                    .align(Alignment.Center)
                    .fillMaxHeight()
            )

            for (component in components) {
                when (component) {
                    is PlayerLike -> {
                        val info = getPlayerInfo(component, content)
                        PlayerPiece(
                            player = info,
                            modifier = Modifier.align(info.pos.toBiasAlignment())
                        )
                    }

                    is BallComponent -> BallPiece(
                        modifier = Modifier.align(component.pos.toPos().toBiasAlignment())
                    )
                }
            }
        }
    }

}