package com.iqball.app.geo

import androidx.compose.ui.BiasAlignment
import kotlin.math.sqrt

typealias Pos = Vector

data class Vector(val x: Double, val y: Double) {
    fun toBiasAlignment(): BiasAlignment =
        BiasAlignment((x * 2 - 1).toFloat(), (y * 2 - 1).toFloat())

    infix operator fun minus(other: Vector) = Vector(x - other.x, y - other.y)
    infix operator fun plus(other: Vector) = Vector(x + other.x, y + other.y)

    fun distanceWith(other: Vector) =
        sqrt(((x - other.x) * (x - other.x) + (y - other.y) * (y - other.y)))

    fun norm() = NullVector.distanceWith(this)

    fun posWithinArea(area: Area) = Vector(x * area.width, y * area.height)
    fun ratioWithinArea(area: Area) =
        Vector((x - area.pos.x) * area.width, (y - area.pos.y) * area.height)
}

val NullVector = Vector(.0, .0)